##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool interface definitions

$Id: interfaces.py,v 1.4 2004/06/28 19:40:25 ctheune Exp $"""
    
from Interface import Interface, Attribute

class ILinkCheckerTool(Interface):

    maximumCacheAge = Attribute("Integer that defines the age of valid information in seconds.")
    gracefulUpdateAge = Attribute("Integer that defines the age when an update is desirable.")

    def getLinkId(url):
        """Return an id that identifies this link within the linkchecker."""

    def getInfoForURL(url):
        """Get an info object for this url."""

    def registerRetriever(portal_type, function_ref):
        """Registers the function_ref to be called on objects
           of portal_type.

           The signature of function_ref must be accepting a single object
           which is the context. The return value is a list of
           ILinkInformation objects.  """

    def checkLink(url):
        """Checks the given URL. 
        
        This result may be cached to allow intelligent load balancing for the
        blocking calls. Returns a boolean value which means true for a link
        that is online and false for a link that is offline."""

    def showLinks():
        """Show the information about the known links.

        Doesn't trigger updates or anything."""

    def checkAllLinks():
        """Schedules checks for all links.

        Takes all links an puts them in the queue."""

    def findLinks():
        """Find new links in the site.

        Finds new links from all objects that are retrievable and
        store them in the cache. No link check is done here."""
        
    def setMaximumCacheAge(age):
        """Sets the maximum cache age."""

    def setGracefulUpdateAge(age):
        """Sets the graceful update age.

        Assures that the graceful update age is lower than the
        maximum cache age."""
 
    def setDefaultURIPrefix(defaultURIPrefix):
        """Set the default URI prefix that will be appended to
           relative URLs.

           If set to a <false> value, it will prepend REQUEST.URL0."""

    def activateWorkerThread():
        """Activate the worker thread.

        Starts the worker thread (if it's not running already)."""

    def runMailNotification():
        """Runs the mail notification code to create the mails
           for every user."""

    # Convenience methods for creating UI stuff

    def getLinksForObject(object):
        """Returns a list of brains (ZCatalog result set) referencing
           ILinkInformation objects that correspond to the given object."""
    
class ILinkInformation(Interface):
    """Information about a link."""

    objects = Attribute("Contains a list of UIDs for objects containing this link.")
    url = Attribute("Contains the URL the link refers to.")
    lastcheck = Attribute("DateTime when the last check was performed.") 
    lastupdate = Attribute("DateTime when the last status change ocurred.")
    laststate = Attribute("The last state the link was in before switching to the current.")
    state = Attribute("The current state of the link. Can be one of ['red', " \
                      "'green', 'orange', 'grey'].")
    reason = Attribute("A verbose reason describing the error")

    def addObject(object):
        """Add this uid to the object list."""
        
    def removeObject(object):
        """Remove this uid from the object list."""

    def getObjects():
        """Returns a list of objects referencing this link."""

    def updateStatus(state, reason):
        """Updates the status information for this link."""

class ILinkChecker(Interface):
    """A class that implements a checking routine to check a specific protocol."""
    
    def check(info):
        """Performs a check on this ILinkInfo object."""
        pass

class ILinkRetriever(Interface):
    """A class that implements a routine to find all links from a given object."""

    def __call__(self, linkchecker, object):
        """Finds all links from the object.

        Returns a list of ILinkInfo objects."""
