##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool - protocol checking functions

$Id: protocols.py,v 1.7 2004/06/03 20:49:32 ctheune Exp $"""

import urllib, httpclient
from httplib import ResponseNotReady
from httpcodes import http_codes
from Interface import Interface

import time
import socket
import ftplib

import zLOG

# CMFLinkChecker imports
from interfaces import ILinkChecker
from timeout_socket import Timeout

class FTPChecker:

    __implements__ = (ILinkChecker,)

    def check(self, info):
        protocol, url = urllib.splittype(info.url)

        assert protocol == "ftp", "FTPChecker only supports ftp links"

        host, path = urllib.splithost(url)
        user, host = urllib.splituser(host)
        if user:
                user, password = urllib.splitpasswd(user)
        else:
                user, password = "anonymous", "cmflinkchecker@gocept.com"
        host, port = urllib.splitport(host)

        # Open connection
        try:
            connection = ftplib.FTP(host, user, password)
        except ftplib.all_errors, exc_info:
            info.updateStatus("red", exc_info[1])
            return

        # Go sure we can get to the data connection, so we try passive mode
        connection.makepasv()

        # Traverse to the correct directory (start not from /, but the home directory, to comply to
        # Internet Explorer FTP behaviour
        path = path.split("/")
        while len(path) > 1:
            step = path.pop(0)
            try:
                connection.cwd(step)
            except ftplib.all_errors, exc_info:
                info.updateStatus("red", exc_info[1])
                return 

        file = path.pop()

        # Check if the file exists
        try:
            if not file in connection.nlst():
                info.updateStatus("red", "File not found")
                return
        except ftplib.all_errors, exc_info:
                info.updateStatus("red", exc_info)

        # Try to start downloada
        def pseudo_callback(block):
            raise "LCCallBackFlag"

        # We try to transfer as few data as possible, maybe just one byte

        rest = connection.size(file)-1
        
        while 1:
            try:
                connection.retrbinary("RETR %s" % file, pseudo_callback, rest=rest)
            except "LCCallBackFlag":
                info.updateStatus("green", "OK")
                break
            except ftplib.error_perm, response:
                if response[0].startswith("502"):       # this is the REST, try again without
                    if rest is not None:
                        rest = None
                    else:
                        info.updateStatus("red", response[1])
                        break
                else:
                    info.updateStatus("red", response[1])
            except Exception, exc_info:
                info.updateStatus("red", exc_info)
                break
        
class HTTPChecker:

    __implements__ = (ILinkChecker,)

    def check(self, info):
        protocol, url = urllib.splittype(info.url)

        # get a connection to the server
        host, path = urllib.splithost(url)
        if host is None:
            info.updateStatus("red", "Invalid Link: No hostname defined")
            return

        host, port = urllib.splitnport(host, 80)
        
        conn = httpclient.myHTTPConnection(host, port, 10)

        starttime = time.time()
        # request the link
        try:
            conn.request("GET", path, headers={"User-Agent": "CMFLinkChecker 1.0rc7", 
                                                "From": "%s <%s>" % (info.email_from_name, info.email_from_address)})
        except socket.error, message:
            if message[0] == -3:
                info.updateStatus("orange", message[1])
            else:
                info.updateStatus("red", "Socket Error: %s" % message[1])
            return
        except Timeout:
            info.updateStatus("red", "Timeout")
            return
        except:
            import sys
            a,b,c = sys.exc_info()
            info.updateStatus("red", b)
            return 

        # look into the response
        response = None
        while not response:
            try:
                now = time.time()
                response = conn.getresponse()
            except ResponseNotReady:
                if now - starttime > 4:
                    break
            except IOError:
                break
            time.sleep(0.2)

        conn.close()
        if response is not None:
            response.close()

        # what if we didn't got a response?
        if not response:
            info.updateStatus("red", "No response from server (timeout)")
            return

        if not http_codes.has_key(response.status):
            info.updateStatus("grey", "unknown HTTP response")
            return
        info.updateStatus(*http_codes.get(response.status))

class UnsupportedChecker:

    __implements__ = (ILinkChecker,)

    def check(self, info):
        info.updateStatus("grey", "Protocol not supported by CMFLinkChecker")

registry = {}

registry['ftp'] = FTPChecker()
registry['http'] = HTTPChecker()
registry['unsupported'] = UnsupportedChecker()

import socket
if hasattr(socket, "ssl"):
    registry['https'] = HTTPChecker()
else:
    zLOG.LOG("CMFLinkChecker", zLOG.WARNING, "HTTPS-Plugin not available", 
             "\tPlease compile your Python with SSL support to be able to " \
             "check HTTPS links.\n")
