##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool - link retriever functions

$Id: retrievers.py,v 1.11 2004/07/02 15:48:42 ctheune Exp $"""

# Plone / CMF imports
try:
    from Products.CMFDefault.Link import Link
except ImportError:
    Link = None

try:
    from Products.CMFDefault.Document import Document
except ImportError:
    Document = None

try:
    from Products.CMFCalendar.Event import Event
except ImportError:
    Event = None

# Zope imports
from StructuredText import Basic
from StructuredText.DocumentClass import StructuredTextLink, DocumentClass

# Python imports
from sgmllib import SGMLParseError
import formatter

# Sibling imports
from LinkInfo import LinkInfo
import lchtmllib
from utils import resolveRelativeLink

def retrieverLink(linkchecker, object):
    """Usable for CMFDefault.Link objects."""
    url = resolveRelativeLink(object.getRemoteUrl(), object)
    info = linkchecker.getInfoForURL(url)
    info.addObject(object)
    
def retrieverEvent(linkchecker, object):
    """Usable for CMFDefault.Event objects."""
    url = resolveRelativeLink(object.event_url, object)
    info = linkchecker.getInfoForURL(url)
    info.addObject(object)
    
def retrieverDocument(linkchecker, object):
    """Usable for CMFDefault.Document objects (in HTML and STX mode)"""
    # Handle document modes
    if object.text_format == 'structured-text':
        links = retrieveSTX(object.text)
    elif object.text_format == 'html':
        links = retrieveHTML(object.text)
    else:
        links = []

    # Handle link registration
    for link in links:
        link = resolveRelativeLink(link, object)
        info = linkchecker.getInfoForURL(link)
        info.addObject(object)

def retrieveSTX(text):
    """Retrieve links from STX data."""
    data = Basic(text)
    data = DocumentClass()(data)
    
    stack = data.getChildren()
    links = []
    while stack:
        element = stack.pop()
        if isinstance(element, StructuredTextLink):
            # Handle images seperately
            element = element.href
            if element.startswith('img:'):
                element = element[4:]
            links.append(element)
        try:
            stack.extend(element.getChildren())
        except AttributeError:
            pass

    return links

def retrieveHTML(text):
    parser = lchtmllib.LCHTMLParser(formatter.NullFormatter())
    try:
        parser.feed(text)
    except SGMLParseError:
        # SGMLLib seems to die on bad HTML sometimes. (At least with python2.1)
        pass
    parser.close()
    return tuple(parser.anchorlist) + tuple(parser.imglist)

        
# simple registries

retrievers = {}

if Link:
    retrievers['Link'] = retrieverLink

if Document:
    retrievers['Document'] = retrieverDocument
    retrievers['NewsItem'] = retrieverDocument

if Event:
    retrievers['Event'] = retrieverEvent
