# Copyright (c) 2004 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: test_linkchecker.py,v 1.2 2004/07/01 06:52:36 ctheune Exp $

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest
import time, Queue

from Testing import ZopeTestCase
ZopeTestCase.installProduct("CMFLinkChecker")

from Products.Archetypes.tests.common import *
from Products.Archetypes.tests.utils import *

from Products.Archetypes.tests.test_sitepolicy import makeContent
from Products.CMFCore.utils import getToolByName

from Products.CMFLinkChecker.interfaces import ILinkInformation
from Products.CMFLinkChecker.LinkCheckerTool import LinkSchedulerThread

class CMFLinkCheckerTest(ArcheSiteTestCase):

    def afterSetUp(self):
        ArcheSiteTestCase.afterSetUp(self)
        user = self.getManagerUser()
        newSecurityManager(None, user)

        portal = self.getPortal()
        qi = getToolByName(portal, 'portal_quickinstaller')
        qi.installProduct('CMFLinkChecker')

    def test_link_registration(self):
        doc = makeContent(self.getPortal(), portal_type="Document", id="asdf")

        lc = getToolByName(self.getPortal(), 'portal_linkchecker')
        links = ["http://www.gocept.com", 
                 "http://www.asfdhasdkjfhasdkjhfkjsda.org", 
                 "http://www.zope.org", "http://www.microsoft.com"]

        for link in links:
            info = lc.getInfoForURL(link)
            info.addObject(doc)

        # Check for the links the object was registered for
        doc_links = lc.getLinksForObject(doc)
        self.assertEqual(len(links), len(doc_links))

        for l in doc_links:
            self.failUnless(l.url in links)


    def test_crawling(self):
        lc = getToolByName(self.getPortal(), 'portal_linkchecker')
        
        links = ["http://www.google.de", "http://www.gocept.com", 
            "http://www.arcor.de", "http://www.gocept.com"]
        docs = []
        
        for x in links:
            id = self.getPortal().generateUniqueId()
            doc = makeContent(self.getPortal(), portal_type="Document", id=id)
            doc.edit('html', '<a href="%s">asdf</a>' % x)
            docs.append(doc)

        lc.findLinks()

        self.assertEqual(lc.countLinks(), 3)    # Doubles only counted once

        for doc in docs:
            l = lc.getLinksForObject(doc)
            self.assertEqual(len(l), 1)
            l = l[0].getObject()
            self.failUnless(ILinkInformation.isImplementedBy(l))
            self.failUnless(l.url in links)
            
            l = lc.getLinksForObject(doc, nobrains=True)
            self.assertEqual(len(l), 1)
            self.failUnless(ILinkInformation.isImplementedBy(l[0]))
            self.failUnless(l[0].url in links)

    #def test_scheduler(self):
    #    return
    #    lc = getToolByName(self.getPortal(), 'portal_linkchecker')
    #
    #    period = 60**-2 * 30    # 30 seconds period in the scheduler
    #    lc.setPeriod(period)
    #
    #    queue = Queue.Queue(0)
    #
    #    scheduler = LinkSchedulerThread(queue, lc.getPhysicalPath())
    #
    #    links = ["http://www.google.de", "http://www.gocept.com", 
    #        "http://www.arcor.de", "http://www.gocept.com"]
    #    docs = []
    #    
    #    for x in links:
    #        id = self.getPortal().generateUniqueId()
    #        doc = makeContent(self.getPortal(), portal_type="Document", id=id)
    #        doc.edit('html', '<a href="%s">asdf</a>' % x)
    #
    #    scheduler.start()
    #    time.sleep(30)
    #    scheduler.stop()
    #    print "waiting for thread to exit"
    #    scheduler.join()
    #    print "done"
    #
    #    self.assertEqual(queue.qsize(), 4)
    #    while queue.qsize():
    #        xxx, key = queue.get()
    #        info = getattr(lc, key)
    #        self.failUnless(info.url in links)


    #def test_checker(self):
    #    return
    #    self.failUnless("Not Implemented" == 1)

    #def test_statistics(self):
    #    return
    #    self.failUnless("Not Implemented" == 1)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CMFLinkCheckerTest))
    return suite

def main():
    from hotshot import Profile
    p = Profile('paras.hotshot')
    unittest.TextTestRunner(verbosity=2).run(test_suite())

if __name__ == '__main__':
    try:
        # if we have hotshot just profile everyting.
        from hotshot import Profile
        p = Profile('editorsupport.hotshot')
        p.runcall(framework)
    except ImportError:
        framework()
else:
    # While framework.py provides its own test_suite()
    # method the testrunner utility does not.
    main()

