# Copyright (c) 2004 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: test_retrievers.py,v 1.1 2004/06/28 19:40:25 ctheune Exp $

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest

from Testing import ZopeTestCase
ZopeTestCase.installProduct("CMFLinkChecker")

from Products.Archetypes.tests.common import *
from Products.Archetypes.tests.utils import *

from Products.Archetypes.tests.test_sitepolicy import makeContent
from Products.CMFCore.utils import getToolByName

from Products.CMFLinkChecker.retrievers import retrieverEvent, retrieverLink, \
        retrieverDocument

class CMFLinkCheckerTest(ArcheSiteTestCase):

    def afterSetUp(self):
        ArcheSiteTestCase.afterSetUp(self)

        user = self.getManagerUser()
        newSecurityManager(None, user)

        portal = self.getPortal()
        qi = getToolByName(portal, 'portal_quickinstaller')
        qi.installProduct('CMFLinkChecker')

    def test_event_retriever(self):
        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://asdf.org"
        event = makeContent(self.getPortal(), portal_type="Event", id="asdf")
        event.edit(event_url=url)

        retrieverEvent(lc, event)

        links = lc.getLinksForObject(event)

        self.assertEqual(len(links), 1)
        self.assertEqual(links[0].url, url)

    def test_link_retriever(self):
        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://www.asdf.org/asdf?asdf"
        ob = makeContent(self.getPortal(), portal_type="Link", id="asdf")
        ob.edit(url)

        retrieverLink(lc, ob)

        links = lc.getLinksForObject(ob)

        self.assertEqual(len(links), 1)
        self.assertEqual(links[0].url, url)

    def test_document_html_retriever(self):
        links = ["http://www.asdf.org", "http://www.bauhaus.de", "ftp://www.goodbye.de", "http://www.google.de/img.png"]

        text = " ".join( [ """<a href="%s">asdf</a>""" % x for x in links[:-1] ])
        text += """<img src="%s"/>""" % links[-1]

        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://www.asdf.org/asdf?asdf"
        ob = makeContent(self.getPortal(), portal_type="Document", id="asdf")
        ob.edit("html", text)

        retrieverDocument(lc, ob)

        links_ob = lc.getLinksForObject(ob)

        self.assertEqual(len(links), len(links_ob))
        for x in links_ob:
            self.failUnless(x.url in links)

    def test_document_stx_retriever(self):
        links = ["http://www.asdf.org", "http://www.bauhaus.de", "ftp://www.goodbye.de", "http://www.google.de/img.png"]

        text = " ".join( [ """"asdf":%s\n""" % x for x in links ])

        lc = getToolByName(self.getPortal(), 'portal_linkchecker')

        url = "http://www.asdf.org/asdf?asdf"
        ob = makeContent(self.getPortal(), portal_type="Document", id="asdf")
        ob.edit("stx", text)

        retrieverDocument(lc, ob)

        links_ob = lc.getLinksForObject(ob)

        self.assertEqual(len(links), len(links_ob))
        for x in links_ob:
            self.failUnless(x.url in links)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CMFLinkCheckerTest))
    return suite

def main():
    from hotshot import Profile
    p = Profile('paras.hotshot')
    unittest.TextTestRunner(verbosity=2).run(test_suite())

if __name__ == '__main__':
    try:
        # if we have hotshot just profile everyting.
        from hotshot import Profile
        p = Profile('editorsupport.hotshot')
        p.runcall(framework)
    except ImportError:
        framework()
else:
    # While framework.py provides its own test_suite()
    # method the testrunner utility does not.
    main()

